@@ -2,20 +2,16 @@
 // Copyright (C) 2016 Intel Corporation.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#ifdef QT_NO_DEBUG
-#undef QT_NO_DEBUG
-#endif
-#ifdef qDebug
-#undef qDebug
-#endif
-
 #include "qdebug.h"
 #include "private/qdebug_p.h"
 #include "qmetaobject.h"
+#include <private/qlogging_p.h>
 #include <private/qtextstream_p.h>
 #include <private/qtools_p.h>
 
+#include <array>
 #include <q20chrono.h>
+#include <cstdio>
 
 QT_BEGIN_NAMESPACE
 
@@ -156,15 +152,15 @@ QByteArray QtDebugUtils::toPrintable(const char *data, qint64 len, qsizetype max
 
     Flushes any pending data to be written and destroys the debug stream.
 */
-// Has been defined in the header / inlined before Qt 5.4
 QDebug::~QDebug()
 {
     if (stream && !--stream->ref) {
         if (stream->space && stream->buffer.endsWith(u' '))
             stream->buffer.chop(1);
         if (stream->message_output) {
+            QInternalMessageLogContext ctxt(stream->context);
             qt_message_output(stream->type,
-                              stream->context,
+                              ctxt,
                               stream->buffer);
         }
         delete stream;
@@ -322,7 +318,7 @@ void QDebug::putString(const QChar *begin, size_t length)
         // we'll reset the QTextStream formatting mechanisms, so save the state
         QDebugStateSaver saver(*this);
         stream->ts.d_ptr->params.reset();
-        putEscapedString(stream->ts.d_ptr.data(), reinterpret_cast<const char16_t *>(begin), length);
+        putEscapedString(stream->ts.d_ptr.get(), reinterpret_cast<const char16_t *>(begin), length);
     }
 }
 
@@ -342,7 +338,7 @@ void QDebug::putByteArray(const char *begin, size_t length, Latin1Content conten
         // we'll reset the QTextStream formatting mechanisms, so save the state
         QDebugStateSaver saver(*this);
         stream->ts.d_ptr->params.reset();
-        putEscapedString(stream->ts.d_ptr.data(), reinterpret_cast<const uchar *>(begin),
+        putEscapedString(stream->ts.d_ptr.get(), reinterpret_cast<const uchar *>(begin),
                          length, content == ContainsLatin1);
     }
 }
@@ -411,9 +407,9 @@ static QByteArray timeUnit(qint64 num, qint64 den)
     };
     auto appendNumber = [&](qint64 value) {
         if (value >= 10'000 && (value % 1000) == 0)
-            len += qsnprintf(buf + len, sizeof(buf) - len, "%.6g", double(value));  // "1e+06"
+            len += std::snprintf(buf + len, sizeof(buf) - len, "%.6g", double(value));  // "1e+06"
         else
-            len += qsnprintf(buf + len, sizeof(buf) - len, "%lld", value);
+            len += std::snprintf(buf + len, sizeof(buf) - len, "%lld", value);
     };
     appendChar('[');
     appendNumber(num);
@@ -436,12 +432,131 @@ void QDebug::putTimeUnit(qint64 num, qint64 den)
     stream->ts << timeUnit(num, den); // ### optimize
 }
 
+namespace {
+
+#ifdef QT_SUPPORTS_INT128
+
+constexpr char Q_INT128_MIN_STR[] = "-170141183460469231731687303715884105728";
+
+constexpr int Int128BufferSize = sizeof(Q_INT128_MIN_STR);
+using Int128Buffer = std::array<char, Int128BufferSize>;
+                                           // numeric_limits<qint128>::digits10 may not exist
+
+static char *i128ToStringHelper(Int128Buffer &buffer, quint128 n)
+{
+    auto dst = buffer.data() + buffer.size();
+    *--dst = '\0'; // NUL-terminate
+    if (n == 0) {
+        *--dst = '0'; // and done
+    } else {
+        while (n != 0) {
+            *--dst = "0123456789"[n % 10];
+            n /= 10;
+        }
+    }
+    return dst;
+}
+#endif // QT_SUPPORTS_INT128
+
+[[maybe_unused]]
+static const char *int128Warning()
+{
+    const char *msg = "Qt was not compiled with int128 support.";
+    qWarning("%s", msg);
+    return msg;
+}
+
+} // unnamed namespace
+
+/*!
+    \since 6.7
+    \internal
+    Helper to the qint128 debug streaming output.
+ */
+void QDebug::putInt128([[maybe_unused]] const void *p)
+{
+#ifdef QT_SUPPORTS_INT128
+    Q_ASSERT(p);
+    qint128 i;
+    memcpy(&i, p, sizeof(i)); // alignment paranoia
+    if (i == Q_INT128_MIN) {
+        // -i is not representable, hardcode the result:
+        stream->ts << Q_INT128_MIN_STR;
+    } else {
+        Int128Buffer buffer;
+        auto dst = i128ToStringHelper(buffer, i < 0 ? -i : i);
+        if (i < 0)
+            *--dst = '-';
+        stream->ts << dst;
+    }
+    return;
+#endif // QT_SUPPORTS_INT128
+    stream->ts << int128Warning();
+}
+
+/*!
+    \since 6.7
+    \internal
+    Helper to the quint128 debug streaming output.
+ */
+void QDebug::putUInt128([[maybe_unused]] const void *p)
+{
+#ifdef QT_SUPPORTS_INT128
+    Q_ASSERT(p);
+    quint128 i;
+    memcpy(&i, p, sizeof(i)); // alignment paranoia
+    Int128Buffer buffer;
+    stream->ts << i128ToStringHelper(buffer, i);
+    return;
+#endif // QT_SUPPORTS_INT128
+    stream->ts << int128Warning();
+}
+
+/*!
+    \since 6.9
+    \internal
+    Helper to the <Std/Qt>::<>_ordering debug output.
+    It generates the string in following format:
+    <Qt/Std>::<weak/partial/strong>_ordering::<less/equal/greater/unordered>
+ */
+void QDebug::putQtOrdering(QtOrderingPrivate::QtOrderingTypeFlag flags, Qt::partial_ordering order)
+{
+    using QtOrderingPrivate::QtOrderingType;
+    std::string result;
+    if ((flags & QtOrderingType::StdOrder) == QtOrderingType::StdOrder)
+        result += "std";
+    else if ((flags & QtOrderingType::QtOrder) == QtOrderingType::QtOrder)
+        result += "Qt";
+
+    result += "::";
+    const bool isStrong = ((flags & QtOrderingType::Strong) == QtOrderingType::Strong);
+    if (isStrong)
+        result += "strong";
+    else if ((flags & QtOrderingType::Weak) == QtOrderingType::Weak)
+        result += "weak";
+    else if ((flags & QtOrderingType::Partial) == QtOrderingType::Partial)
+        result += "partial";
+    result += "_ordering::";
+
+    if (order == Qt::partial_ordering::equivalent) {
+        if (isStrong)
+            result += "equal";
+        else
+            result += "equivalent";
+    } else if (order == Qt::partial_ordering::greater) {
+        result += "greater";
+    } else if (order == Qt::partial_ordering::less) {
+        result += "less";
+    } else {
+        result += "unordered";
+    }
+    stream->ts << result.data();
+}
+
 /*!
     \fn QDebug::swap(QDebug &other)
     \since 5.0
-
-    Swaps this debug stream instance with \a other. This function is
-    very fast and never fails.
+    \memberswap{debug stream instance}
 */
 
 /*!
@@ -511,6 +626,29 @@ QDebug &QDebug::resetFormat()
 */
 
 
+/*!
+    \fn bool QDebug::quoteStrings() const
+    \since 6.7
+
+    Returns \c true if this QDebug instance will quote strings streamed into
+    it (which is the default).
+
+    \sa QDebugStateSaver, quote(), noquote(), setQuoteStrings()
+*/
+
+/*!
+    \fn void QDebug::setQuoteStrings(bool b)
+    \since 6.7
+
+    Enables quoting of strings streamed into this QDebug instance if \a b is
+    \c true; otherwise quoting is disabled.
+
+    The default is to quote strings.
+
+    \sa QDebugStateSaver, quote(), noquote(), quoteStrings()
+*/
+
+
 /*!
     \fn QDebug &QDebug::quote()
     \since 5.4
@@ -849,6 +987,14 @@ QDebug &QDebug::resetFormat()
     \internal
 */
 
+/*!
+    \fn template <typename T, QDebug::if_ordering_type<T>> QDebug::operator<<(QDebug debug, T t)
+    \since 6.9
+    Prints the Qt or std ordering value \a t to the \a debug object.
+
+    \constraints \c T is one of <Qt/Std>::<weak/partial/strong>_ordering.
+*/
+
 /*!
     \since 6.5
     \fn template <typename Char, typename...Args> QDebug &QDebug::operator<<(const std::basic_string<Char, Args...> &s)
@@ -880,12 +1026,90 @@ QDebug &QDebug::resetFormat()
 */
 
 /*!
-    \fn template <class T> QString QDebug::toString(T &&object)
+    \fn template <typename T, QDebug::if_qint128<T>> QDebug::operator<<(T i)
+    \fn template <typename T, QDebug::if_quint128<T>> QDebug::operator<<(T i)
+    \since 6.7
+
+    Prints the textual representation of the 128-bit integer \a i.
+
+    \note This operator is only available if Qt supports 128-bit integer types.
+    If 128-bit integer types are available in your build, but the Qt libraries
+    were compiled without, the operator will print a warning instead.
+
+    \note Because the operator is a function template, no implicit conversions
+    are performed on its argument. It must be exactly qint128/quint128.
+
+    \sa QT_SUPPORTS_INT128
+*/
+
+/*!
+    \fn template <class T> QString QDebug::toString(const T &object)
     \since 6.0
 
     \include qdebug-toString.qdocinc
+
+    \sa toBytes()
 */
 
+/*! \internal */
+QString QDebug::toStringImpl(StreamTypeErased s, const void *obj)
+{
+    QString result;
+    {
+        QDebug d(&result);
+        s(d.nospace(), obj);
+    }
+    return result;
+}
+
+/*!
+    \fn template <class T> QByteArray QDebug::toBytes(const T &object)
+    \since 6.9
+
+    This is equivalent to passing \a object to
+    \c{QDebug::toString(object).toUtf8()}, but more efficient.
+
+    \sa toString()
+*/
+
+/*! \internal */
+QByteArray QDebug::toBytesImpl(StreamTypeErased s, const void *obj)
+{
+    QByteArray result;
+    {
+        QDebug d(&result);
+        s(d.nospace(), obj);
+    }
+    return result;
+}
+
+/*!
+    \internal
+    \since 6.9
+
+    Outputs a heterogeneous product type (pair, tuple, or anything that
+    implements the Tuple Protocol). The class name is described by "\a ns
+    \c{::} \a what", while the addresses of the \a n elements are stored in the
+    array \a data. The formatters are stored in the array \a ops.
+
+    If \a ns is empty, only \a what is used.
+*/
+QDebug &QDebug::putTupleLikeImplImpl(const char *ns, const char *what,
+                                     size_t n, StreamTypeErased *ops, const void **data)
+{
+    const QDebugStateSaver saver(*this);
+    nospace();
+    if (ns && *ns)
+        *this << ns << "::";
+    *this << what << '(';
+    while (n--) {
+        (*ops++)(*this, *data++);
+        if (n)
+            *this << ", ";
+    }
+    return *this << ')';
+}
+
 /*!
     \fn template <class T> QDebug operator<<(QDebug debug, const QList<T> &list)
     \relates QDebug
@@ -921,6 +1145,15 @@ QDebug &QDebug::resetFormat()
     support streaming into QDebug.
 */
 
+/*!
+    \fn template <typename T, std::size_t N> QDebug operator<<(QDebug debug, const std::array<T, N> &array)
+    \relates QDebug
+    \since 6.9
+
+    Writes the contents of \a array to \a debug. \c T needs to
+    support streaming into QDebug.
+*/
+
 /*!
     \fn template <typename T> QDebug operator<<(QDebug debug, const QSet<T> &set)
     \relates QDebug
@@ -963,6 +1196,43 @@ QDebug &QDebug::resetFormat()
     \c T need to support streaming into QDebug.
 */
 
+/*!
+    \fn template <typename Key, typename Compare, typename Alloc> QDebug operator<<(QDebug debug, const std::multiset<Key, Compare, Alloc> &multiset)
+    \relates QDebug
+    \since 6.9
+
+    Writes the contents of \a multiset to \a debug. The \c Key type
+    needs to support streaming into QDebug.
+*/
+
+/*!
+    \fn template <typename Key, typename Compare, typename Alloc> QDebug operator<<(QDebug debug, const std::set<Key, Compare, Alloc> &set)
+    \relates QDebug
+    \since 6.9
+
+    Writes the contents of \a set to \a debug. The \c Key type
+    needs to support streaming into QDebug.
+*/
+
+/*!
+    \fn template <typename Key, typename T, typename Hash, typename KeyEqual, typename Alloc> QDebug operator<<(QDebug debug, const std::unordered_map<Key, T, Hash, KeyEqual, Alloc> &map)
+    \relates QDebug
+    \since 6.9
+
+ Writes the contents of \a map to \a debug. Both \c Key and
+ \c T need to support streaming into QDebug.
+*/
+
+/*!
+    \fn template <typename Key, typename Hash, typename KeyEqual, typename Alloc> QDebug operator<<(QDebug debug, const std::unordered_set<Key, Hash, KeyEqual, Alloc> &unordered_set)
+    \relates QDebug
+    \since 6.9
+
+Writes the contents of \a unordered_set to \a debug. The \c Key type
+needs to support streaming into QDebug.
+*/
+
+
 /*!
     \fn template <class Key, class T> QDebug operator<<(QDebug debug, const QHash<Key, T> &hash)
     \relates QDebug
@@ -980,11 +1250,11 @@ QDebug &QDebug::resetFormat()
 */
 
 /*!
-    \fn template <class T1, class T2> QDebug operator<<(QDebug debug, const QPair<T1, T2> &pair)
-    \relates QDebug
+    \fn template <class...Ts, QDebug::if_streamable<Ts...>> QDebug &QDebug::operator<<(const std::tuple<Ts...> &tuple)
+    \since 6.9
 
-    Writes the contents of \a pair to \a debug. Both \c T1 and
-    \c T2 need to support streaming into QDebug.
+    Writes the contents of \a tuple to the stream. All \c Ts... need to support
+    streaming into QDebug.
 */
 
 /*!
@@ -995,6 +1265,14 @@ QDebug &QDebug::resetFormat()
     \c T2 need to support streaming into QDebug.
 */
 
+/*!
+    \since 6.7
+    \fn template <class T, QDebug::if_streamable<T>> QDebug::operator<<(const std::optional<T> &opt)
+
+    Writes the contents of \a opt (or \c nullopt if not set) to this stream.
+    \c T needs to support streaming into QDebug.
+*/
+
 /*!
     \fn template <typename T> QDebug operator<<(QDebug debug, const QContiguousCache<T> &cache)
     \relates QDebug
@@ -1027,6 +1305,13 @@ QDebug &QDebug::resetFormat()
   \internal
  */
 
+/*!
+    \since 6.7
+    \fn QDebug &QDebug::operator<<(std::nullopt_t)
+
+    Writes nullopt to the stream.
+*/
+
 /*!
     \class QDebugStateSaver
     \inmodule QtCore
@@ -1115,15 +1400,25 @@ QDebugStateSaver::~QDebugStateSaver()
     \internal
 
     Specialization of the primary template in qdebug.h to out-of-line
-    the common case of QFlags<T>::Int being int.
+    the common case of QFlags<T>::Int being 32-bit.
 
     Just call the generic version so the two don't get out of sync.
 */
-void qt_QMetaEnum_flagDebugOperator(QDebug &debug, size_t sizeofT, int value)
+void qt_QMetaEnum_flagDebugOperator(QDebug &debug, size_t sizeofT, uint value)
 {
-    qt_QMetaEnum_flagDebugOperator<int>(debug, sizeofT, value);
+    qt_QMetaEnum_flagDebugOperator(debug, sizeofT, quint64(value));
 }
 
+/*!
+    \internal
+    Ditto, for 64-bit.
+*/
+void qt_QMetaEnum_flagDebugOperator(QDebug &debug, size_t sizeofT, quint64 value)
+{
+    qt_QMetaEnum_flagDebugOperator<quint64>(debug, sizeofT, value);
+}
+
+
 #ifndef QT_NO_QOBJECT
 /*!
     \internal
@@ -1241,7 +1536,7 @@ QDebug qt_QMetaEnum_flagDebugOperator(QDebug &debug, quint64 value, const QMetaO
         debug << '(';
     }
 
-    debug << me.valueToKeys(static_cast<int>(value));
+    debug << me.valueToKeys(value);
 
     if (enumScope)
         debug << ')';
