@@ -10,14 +10,27 @@
 
 QT_BEGIN_NAMESPACE
 
+/*!
+    \macro QT_NO_QSNPRINTF
+    \since 6.8
+    \relates QByteArray
+
+    Defining this macro removes the availability of the qsnprintf() and
+    qvsnprintf() functions. See the functions' documentation for why you may
+    want to disable them.
+
+    \sa qsnprintf(), qvsnprintf().
+*/
+
+#if QT_DEPRECATED_SINCE(6, 9)
+
 #if !defined(QT_VSNPRINTF) || defined(Q_QDOC)
 
 /*!
+    \fn int qvsnprintf(char *str, size_t n, const char *fmt, va_list ap)
     \relates QByteArray
 
-    \obsolete
-
-    Use C++11's \c{std::vsnprintf()} from \c{<cstdio>} instead.
+    \deprecated [6.9] Use C++11's \c{std::vsnprintf()} from \c{<cstdio>} instead.
 
     A portable \c vsnprintf() function. Will call \c ::vsnprintf(), \c
     ::_vsnprintf(), or \c ::vsnprintf_s depending on the system, or
@@ -40,6 +53,7 @@ QT_BEGIN_NAMESPACE
     \sa qsnprintf(), QString::asprintf()
 */
 
+Q_CORE_EXPORT // QT_NO_QSNPRINTF is in effect
 int qvsnprintf(char *str, size_t n, const char *fmt, va_list ap)
 {
     if (!str || !fmt)
@@ -62,6 +76,7 @@ QT_BEGIN_INCLUDE_NAMESPACE
 #include <stdio.h>
 QT_END_INCLUDE_NAMESPACE
 
+Q_CORE_EXPORT // QT_NO_QSNPRINTF is in effect
 int qvsnprintf(char *str, size_t n, const char *fmt, va_list ap)
 {
     return QT_VSNPRINTF(str, n, fmt, ap);
@@ -70,12 +85,11 @@ int qvsnprintf(char *str, size_t n, const char *fmt, va_list ap)
 #endif
 
 /*!
+    \fn int qsnprintf(char *str, size_t n, const char *fmt, ...)
     \target bytearray-qsnprintf
     \relates QByteArray
 
-    \obsolete
-
-    Use C++11's \c{std::snprintf()} from \c{<cstdio>} instead.
+    \deprecated [6.9] Use C++11's \c{std::snprintf()} from \c{<cstdio>} instead.
 
     A portable snprintf() function, calls qvsnprintf.
 
@@ -89,15 +103,20 @@ int qvsnprintf(char *str, size_t n, const char *fmt, va_list ap)
     \sa qvsnprintf(), QString::asprintf()
 */
 
+Q_CORE_EXPORT // QT_NO_QSNPRINTF is in effect
 int qsnprintf(char *str, size_t n, const char *fmt, ...)
 {
     va_list ap;
     va_start(ap, fmt);
 
+    QT_IGNORE_DEPRECATIONS(
     int ret = qvsnprintf(str, n, fmt, ap);
+    )
     va_end(ap);
 
     return ret;
 }
 
+#endif // QT_DEPRECATED_SINCE(6, 9)
+
 QT_END_NAMESPACE
