@@ -1,5 +1,6 @@
 // Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qplatformdefs.h"
 #include "private/qdatetimeparser_p.h"
@@ -303,6 +304,9 @@ int QDateTimeParser::absoluteMin(int s) const
 
 const QDateTimeParser::SectionNode &QDateTimeParser::sectionNode(int sectionIndex) const
 {
+    static constexpr SectionNode first{FirstSection, 0, -1, 0};
+    static constexpr SectionNode last{LastSection, -1, -1, 0};
+    static constexpr SectionNode none{NoSection, -1, -1, 0};
     if (sectionIndex < 0) {
         switch (sectionIndex) {
         case FirstSectionIndex:
@@ -338,7 +342,7 @@ int QDateTimeParser::sectionPos(int sectionIndex) const
     return sectionPos(sectionNode(sectionIndex));
 }
 
-int QDateTimeParser::sectionPos(const SectionNode &sn) const
+int QDateTimeParser::sectionPos(SectionNode sn) const
 {
     switch (sn.type) {
     case FirstSection: return 0;
@@ -456,7 +460,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
             case 'h':
                 if (parserType != QMetaType::QDate) {
                     const Section hour = (sect == 'h') ? Hour12Section : Hour24Section;
-                    const SectionNode sn = { hour, i - add, countRepeat(newFormat, i, 2), 0 };
+                    const SectionNode sn{hour, i - add, countRepeat(newFormat, i, 2)};
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += sn.count - 1;
@@ -466,7 +470,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                 break;
             case 'm':
                 if (parserType != QMetaType::QDate) {
-                    const SectionNode sn = { MinuteSection, i - add, countRepeat(newFormat, i, 2), 0 };
+                    const SectionNode sn{MinuteSection, i - add, countRepeat(newFormat, i, 2)};
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += sn.count - 1;
@@ -476,7 +480,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                 break;
             case 's':
                 if (parserType != QMetaType::QDate) {
-                    const SectionNode sn = { SecondSection, i - add, countRepeat(newFormat, i, 2), 0 };
+                    const SectionNode sn{SecondSection, i - add, countRepeat(newFormat, i, 2)};
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += sn.count - 1;
@@ -488,7 +492,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
             case 'z':
                 if (parserType != QMetaType::QDate) {
                     const int repeat = countRepeat(newFormat, i, 3);
-                    const SectionNode sn = { MSecSection, i - add, repeat < 3 ? 1 : 3, 0 };
+                    const SectionNode sn{MSecSection, i - add, repeat < 3 ? 1 : 3};
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += repeat - 1;
@@ -509,7 +513,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                         if (newFormat.at(i) != QLatin1Char(caseOpt == UpperCase ? 'P' : 'p'))
                             caseOpt = NativeCase;
                     }
-                    const SectionNode sn = { AmPmSection, pos, int(caseOpt), 0 };
+                    const SectionNode sn{AmPmSection, pos, int(caseOpt)};
                     newSectionNodes.append(sn);
                     index = i + 1;
                 }
@@ -518,8 +522,8 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                 if (parserType != QMetaType::QTime) {
                     const int repeat = countRepeat(newFormat, i, 4);
                     if (repeat >= 2) {
-                        const SectionNode sn = { repeat == 4 ? YearSection : YearSection2Digits,
-                                                 i - add, repeat == 4 ? 4 : 2, 0 };
+                        const SectionNode sn{repeat == 4 ? YearSection : YearSection2Digits,
+                                             i - add, repeat == 4 ? 4 : 2};
                         newSectionNodes.append(sn);
                         appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                         i += sn.count - 1;
@@ -530,7 +534,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                 break;
             case 'M':
                 if (parserType != QMetaType::QTime) {
-                    const SectionNode sn = { MonthSection, i - add, countRepeat(newFormat, i, 4), 0 };
+                    const SectionNode sn{MonthSection, i - add, countRepeat(newFormat, i, 4)};
                     newSectionNodes.append(sn);
                     newSeparators.append(unquote(newFormat.first(i).sliced(index)));
                     i += sn.count - 1;
@@ -543,7 +547,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                     const int repeat = countRepeat(newFormat, i, 4);
                     const Section sectionType = (repeat == 4 ? DayOfWeekSectionLong
                         : (repeat == 3 ? DayOfWeekSectionShort : DaySection));
-                    const SectionNode sn = { sectionType, i - add, repeat, 0 };
+                    const SectionNode sn{sectionType, i - add, repeat};
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += sn.count - 1;
@@ -553,8 +557,7 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
                 break;
             case 't':
                 if (parserType == QMetaType::QDateTime) {
-                    const SectionNode sn
-                        = { TimeZoneSection, i - add, countRepeat(newFormat, i, 4), 0 };
+                    const SectionNode sn{TimeZoneSection, i - add, countRepeat(newFormat, i, 4)};
                     newSectionNodes.append(sn);
                     appendSeparator(&newSeparators, newFormat, index, i - index, lastQuote);
                     i += sn.count - 1;
@@ -588,7 +591,6 @@ bool QDateTimeParser::parseFormat(QStringView newFormat)
     separators = newSeparators;
     sectionNodes = newSectionNodes;
     display = newDisplay;
-    last.pos = -1;
 
 //     for (int i=0; i<sectionNodes.size(); ++i) {
 //         QDTPDEBUG << sectionNodes.at(i).name() << sectionNodes.at(i).count;
@@ -784,7 +786,7 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
 
     const int sectionmaxsize = sectionMaxSize(sectionIndex);
     const bool negate = (sn.type == YearSection && m_text.size() > offset
-                         && m_text.at(offset) == u'-');
+                         && calendar.isProleptic() && m_text.at(offset) == u'-');
     const int negativeYearOffset = negate ? 1 : 0;
 
     QStringView sectionTextRef =
@@ -945,8 +947,14 @@ QDateTimeParser::parseSection(const QDateTime &currentValue, int sectionIndex, i
                         m_text.insert(offset, QString(missingZeroes, u'0'));
                         ++(const_cast<QDateTimeParser*>(this)->sectionNodes[sectionIndex].zeroesAdded);
                     } else {
-                        result = ParsedSection(Intermediate, lastVal, used);;
+                        result = ParsedSection(Intermediate, lastVal, used);
                     }
+                } else if (!lastVal && !calendar.hasYearZero()
+                           && (sn.type == YearSection
+                               || (sn.type == YearSection2Digits && currentValue.isValid()
+                                   && currentValue.date().year() / 100 == 0))) {
+                    // Year zero prohibited
+                    result = ParsedSection(unfilled ? Acceptable : Invalid, lastVal, used);
                 } else {
                     result = ParsedSection(Acceptable, lastVal, used);
                 }
@@ -1000,7 +1008,7 @@ static int yearInCenturyFrom(int y2d, int baseYear)
   when on valid date is consistent with the data.
 */
 
-static QDate actualDate(QDateTimeParser::Sections known, const QCalendar &calendar, int baseYear,
+static QDate actualDate(QDateTimeParser::Sections known, QCalendar calendar, int baseYear,
                         int year, int year2digits, int month, int day, int dayofweek)
 {
     QDate actual(year, month, day, calendar);
@@ -1265,7 +1273,7 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         if (fixup && sect.state == Intermediate && sect.used < sn.count) {
             const FieldInfo fi = fieldInfo(index);
             if ((fi & (Numeric|FixedWidth)) == (Numeric|FixedWidth)) {
-                const QString newText = QString("%1"_L1).arg(sect.value, sn.count, 10, '0'_L1);
+                const QString newText = QString::asprintf("%0*d", sn.count, sect.value);
                 m_text.replace(pos, sect.used, newText);
                 sect.used = sn.count;
             }
@@ -1350,7 +1358,9 @@ QDateTimeParser::scanString(const QDateTime &defaultValue, bool fixup) const
         if (year % 100 != year2digits && (isSet & YearSection2Digits)) {
             const QDate date = actualDate(isSet, calendar, defaultCenturyStart,
                                           year, year2digits, month, day, dayofweek);
-            if (!(isSet & YearSection)) {
+            if (!date.isValid()) {
+                state = Invalid;
+            } else if (!(isSet & YearSection)) {
                 year = date.year();
             } else {
                 conflicts = true;
@@ -1612,16 +1622,11 @@ QDateTimeParser::parse(const QString &input, int position,
                 }
             }
         } else {
-            if (context == FromString) {
-                // optimization
-                Q_ASSERT(maximum.date().toJulianDay() == 5373484);
-                if (scan.value.date().toJulianDay() > 5373484)
-                    scan.state = Invalid;
-            } else if (scan.value > maximum) {
+            if (scan.value > maximum)
                 scan.state = Invalid;
-            }
 
-            QDTPDEBUG << "not checking intermediate because scanned value is" << scan.value << minimum << maximum;
+            QDTPDEBUG << "not checking intermediate because scanned value is"
+                      << scan.value << minimum << maximum;
         }
     }
 
@@ -1642,7 +1647,7 @@ QDateTimeParser::parse(const QString &input, int position,
   length of overlap in *used (if \a used is non-NULL) and the first entry that
   overlapped this much in *usedText (if \a usedText is non-NULL).
  */
-static int findTextEntry(const QString &text, const ShortVector<QString> &entries, QString *usedText, int *used)
+static int findTextEntry(QStringView text, const ShortVector<QString> &entries, QString *usedText, int *used)
 {
     if (text.isEmpty())
         return -1;
@@ -1679,7 +1684,7 @@ static int findTextEntry(const QString &text, const ShortVector<QString> &entrie
   match. Starting from \a index; str should already by lowered
 */
 
-int QDateTimeParser::findMonth(const QString &str1, int startMonth, int sectionIndex,
+int QDateTimeParser::findMonth(QStringView str, int startMonth, int sectionIndex,
                                int year, QString *usedMonth, int *used) const
 {
     const SectionNode &sn = sectionNode(sectionIndex);
@@ -1695,11 +1700,11 @@ int QDateTimeParser::findMonth(const QString &str1, int startMonth, int sectionI
     for (int month = startMonth; month <= 12; ++month)
         monthNames.append(calendar.monthName(l, month, year, type));
 
-    const int index = findTextEntry(str1, monthNames, usedMonth, used);
+    const int index = findTextEntry(str, monthNames, usedMonth, used);
     return index < 0 ? index : index + startMonth;
 }
 
-int QDateTimeParser::findDay(const QString &str1, int startDay, int sectionIndex, QString *usedDay, int *used) const
+int QDateTimeParser::findDay(QStringView str, int startDay, int sectionIndex, QString *usedDay, int *used) const
 {
     const SectionNode &sn = sectionNode(sectionIndex);
     if (!(sn.type & DaySectionMask)) {
@@ -1714,7 +1719,7 @@ int QDateTimeParser::findDay(const QString &str1, int startDay, int sectionIndex
     for (int day = startDay; day <= 7; ++day)
         daysOfWeek.append(l.dayName(day, type));
 
-    const int index = findTextEntry(str1, daysOfWeek, usedDay, used);
+    const int index = findTextEntry(str, daysOfWeek, usedDay, used);
     return index < 0 ? index : index + startDay;
 }
 
@@ -2316,8 +2321,8 @@ QString QDateTimeParser::getAmPmText(AmPm ap, Case cs) const
     QString raw = ap == AmText ? loc.amText() : loc.pmText();
     switch (cs)
     {
-    case UpperCase: return raw.toUpper();
-    case LowerCase: return raw.toLower();
+    case UpperCase: return std::move(raw).toUpper();
+    case LowerCase: return std::move(raw).toLower();
     case NativeCase: return raw;
     }
     Q_UNREACHABLE_RETURN(raw);
@@ -2325,11 +2330,9 @@ QString QDateTimeParser::getAmPmText(AmPm ap, Case cs) const
 
 /*
   \internal
-
-  I give arg2 preference because arg1 is always a QDateTime.
 */
 
-bool operator==(const QDateTimeParser::SectionNode &s1, const QDateTimeParser::SectionNode &s2)
+bool operator==(QDateTimeParser::SectionNode s1, QDateTimeParser::SectionNode s2)
 {
     return (s1.type == s2.type) && (s1.pos == s2.pos) && (s1.count == s2.count);
 }
@@ -2338,7 +2341,7 @@ bool operator==(const QDateTimeParser::SectionNode &s1, const QDateTimeParser::S
   Sets \a cal as the calendar to use. The default is Gregorian.
 */
 
-void QDateTimeParser::setCalendar(const QCalendar &cal)
+void QDateTimeParser::setCalendar(QCalendar cal)
 {
     calendar = cal;
 }
