@@ -1,55 +1,22 @@
-/****************************************************************************
-**
-** Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2013 Samuel Gaist <samuel.gaist@edeltech.ch>
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "private/qpnghandler_p.h"
 
 #ifndef QT_NO_IMAGEFORMAT_PNG
 #include <qcoreapplication.h>
+#include <qdebug.h>
 #include <qiodevice.h>
 #include <qimage.h>
-#include <qlist.h>
+#include <qloggingcategory.h>
 #include <qvariant.h>
-#include <qvector.h>
 
 #include <private/qimage_p.h> // for qt_getImageText
 
+#include <qcolorspace.h>
+#include <private/qcolorspace_p.h>
+
 #include <png.h>
 #include <pngconf.h>
 
@@ -78,6 +45,8 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 // avoid going through QImage::scanLine() which calls detach
 #define FAST_SCAN_LINE(data, bpl, y) (data + (y) * bpl)
 
@@ -96,9 +65,17 @@ public:
         ReadingEnd,
         Error
     };
+    // Defines the order of how the various ways of setting colorspace overrides each other:
+    enum ColorSpaceState {
+        Undefined = 0,
+        GammaChrm = 1, // gAMA+cHRM chunks
+        Srgb = 2,      // sRGB chunk
+        Icc = 3        // iCCP chunk
+    };
 
     QPngHandlerPrivate(QPngHandler *qq)
-        : gamma(0.0), fileGamma(0.0), quality(50), compression(50), png_ptr(0), info_ptr(0), end_info(0), state(Ready), q(qq)
+        : gamma(0.0), fileGamma(0.0), quality(50), compression(50), colorSpaceState(Undefined),
+          png_ptr(nullptr), info_ptr(nullptr), end_info(nullptr), row_pointers(nullptr), state(Ready), q(qq)
     { }
 
     float gamma;
@@ -106,12 +83,14 @@ public:
     int quality; // quality is used for backward compatibility, maps to compression
     int compression;
     QString description;
-    QSize scaledSize;
     QStringList readTexts;
+    QColorSpace colorSpace;
+    ColorSpaceState colorSpaceState;
 
     png_struct *png_ptr;
     png_info *info_ptr;
     png_info *end_info;
+    png_byte **row_pointers;
 
     bool readPngHeader();
     bool readPngImage(QImage *image);
@@ -119,30 +98,6 @@ public:
 
     QImage::Format readImageFormat();
 
-    struct AllocatedMemoryPointers {
-        AllocatedMemoryPointers()
-            : row_pointers(0), accRow(0), inRow(0), outRow(0)
-        { }
-        void deallocate()
-        {
-            delete [] row_pointers;
-            row_pointers = 0;
-            delete [] accRow;
-            accRow = 0;
-            delete [] inRow;
-            inRow = 0;
-            delete [] outRow;
-            outRow = 0;
-        }
-
-        png_byte **row_pointers;
-        quint32 *accRow;
-        png_byte *inRow;
-        uchar *outRow;
-    };
-
-    AllocatedMemoryPointers amp;
-
     State state;
 
     QPngHandler *q;
@@ -161,7 +116,7 @@ public:
     void setGamma(float);
 
     bool writeImage(const QImage& img, int x, int y);
-    bool writeImage(const QImage& img, volatile int compression_in, const QString &description, int x, int y);
+    bool writeImage(const QImage& img, int compression_in, const QString &description, int x, int y);
     bool writeImage(const QImage& img)
         { return writeImage(img, 0, 0); }
     bool writeImage(const QImage& img, int compression, const QString &description)
@@ -185,7 +140,7 @@ void iod_read_fn(png_structp png_ptr, png_bytep data, png_size_t length)
     QPngHandlerPrivate *d = (QPngHandlerPrivate *)png_get_io_ptr(png_ptr);
     QIODevice *in = d->q->device();
 
-    if (d->state == QPngHandlerPrivate::ReadingEnd && !in->isSequential() && (in->size() - in->pos()) < 4 && length == 4) {
+    if (d->state == QPngHandlerPrivate::ReadingEnd && !in->isSequential() && in->size() > 0 && (in->size() - in->pos()) < 4 && length == 4) {
         // Workaround for certain malformed PNGs that lack the final crc bytes
         uchar endcrc[4] = { 0xae, 0x42, 0x60, 0x82 };
         memcpy(data, endcrc, 4);
@@ -226,22 +181,19 @@ void qpiw_flush_fn(png_structp /* png_ptr */)
 }
 
 static
-void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scaledSize, bool *doScaledRead, float screen_gamma=0.0, float file_gamma=0.0)
+bool setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr)
 {
-    if (screen_gamma != 0.0 && file_gamma != 0.0)
-        png_set_gamma(png_ptr, 1.0f / screen_gamma, file_gamma);
-
-    png_uint_32 width;
-    png_uint_32 height;
+    png_uint_32 width = 0;
+    png_uint_32 height = 0;
     int bit_depth = 0;
     int color_type = 0;
-    png_bytep trans_alpha = 0;
-    png_color_16p trans_color_p = 0;
+    png_bytep trans_alpha = nullptr;
+    png_color_16p trans_color_p = nullptr;
     int num_trans;
-    png_colorp palette = 0;
+    png_colorp palette = nullptr;
     int num_palette;
-    int interlace_method;
-    png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, &interlace_method, 0, 0);
+    png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, nullptr, nullptr, nullptr);
+    QSize size(width, height);
     png_set_interlace_handling(png_ptr);
 
     if (color_type == PNG_COLOR_TYPE_GRAY) {
@@ -249,11 +201,8 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
         if (bit_depth == 1 && png_get_channels(png_ptr, info_ptr) == 1) {
             png_set_invert_mono(png_ptr);
             png_read_update_info(png_ptr, info_ptr);
-            if (image.size() != QSize(width, height) || image.format() != QImage::Format_Mono) {
-                image = QImage(width, height, QImage::Format_Mono);
-                if (image.isNull())
-                    return;
-            }
+            if (!QImageIOHandler::allocateImage(size, QImage::Format_Mono, &image))
+                return false;
             image.setColorCount(2);
             image.setColor(1, qRgb(0,0,0));
             image.setColor(0, qRgb(255,255,255));
@@ -266,6 +215,14 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
                 else if (g == 1)
                     image.setColor(0, qRgba(255, 255, 255, 0));
             }
+        } else if (bit_depth == 16
+                   && png_get_channels(png_ptr, info_ptr) == 1
+                   && !png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) {
+            if (!QImageIOHandler::allocateImage(size, QImage::Format_Grayscale16, &image))
+                return false;
+            png_read_update_info(png_ptr, info_ptr);
+            if (QSysInfo::ByteOrder == QSysInfo::LittleEndian)
+                png_set_swap(png_ptr);
         } else if (bit_depth == 16) {
             bool hasMask = png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS);
             if (!hasMask)
@@ -274,33 +231,23 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
                 png_set_expand(png_ptr);
             png_set_gray_to_rgb(png_ptr);
             QImage::Format format = hasMask ? QImage::Format_RGBA64 : QImage::Format_RGBX64;
-            if (image.size() != QSize(width, height) || image.format() != format) {
-                image = QImage(width, height, format);
-                if (image.isNull())
-                    return;
-            }
+            if (!QImageIOHandler::allocateImage(size, format, &image))
+                return false;
             png_read_update_info(png_ptr, info_ptr);
             if (QSysInfo::ByteOrder == QSysInfo::LittleEndian)
                 png_set_swap(png_ptr);
         } else if (bit_depth == 8 && !png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) {
             png_set_expand(png_ptr);
-            if (image.size() != QSize(width, height) || image.format() != QImage::Format_Grayscale8) {
-                image = QImage(width, height, QImage::Format_Grayscale8);
-                if (image.isNull())
-                    return;
-            }
-
+            if (!QImageIOHandler::allocateImage(size, QImage::Format_Grayscale8, &image))
+                return false;
             png_read_update_info(png_ptr, info_ptr);
         } else {
             if (bit_depth < 8)
                 png_set_packing(png_ptr);
             int ncols = bit_depth < 8 ? 1 << bit_depth : 256;
             png_read_update_info(png_ptr, info_ptr);
-            if (image.size() != QSize(width, height) || image.format() != QImage::Format_Indexed8) {
-                image = QImage(width, height, QImage::Format_Indexed8);
-                if (image.isNull())
-                    return;
-            }
+            if (!QImageIOHandler::allocateImage(size, QImage::Format_Indexed8, &image))
+                return false;
             image.setColorCount(ncols);
             for (int i=0; i<ncols; i++) {
                 int c = i*255/(ncols-1);
@@ -321,13 +268,11 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
         if (bit_depth != 1)
             png_set_packing(png_ptr);
         png_read_update_info(png_ptr, info_ptr);
-        png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, 0, 0, 0);
+        png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, nullptr, nullptr, nullptr);
+        size = QSize(width, height);
         QImage::Format format = bit_depth == 1 ? QImage::Format_Mono : QImage::Format_Indexed8;
-        if (image.size() != QSize(width, height) || image.format() != format) {
-            image = QImage(width, height, format);
-            if (image.isNull())
-                return;
-        }
+        if (!QImageIOHandler::allocateImage(size, format, &image))
+            return false;
         png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette);
         image.setColorCount((format == QImage::Format_Mono) ? 2 : num_palette);
         int i = 0;
@@ -365,11 +310,8 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
         }
         if (!(color_type & PNG_COLOR_MASK_COLOR))
             png_set_gray_to_rgb(png_ptr);
-        if (image.size() != QSize(width, height) || image.format() != format) {
-            image = QImage(width, height, format);
-            if (image.isNull())
-                return;
-        }
+        if (!QImageIOHandler::allocateImage(size, format, &image))
+            return false;
         png_read_update_info(png_ptr, info_ptr);
         if (QSysInfo::ByteOrder == QSysInfo::LittleEndian)
             png_set_swap(png_ptr);
@@ -392,19 +334,8 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
             // We want 4 bytes, but it isn't an alpha channel
             format = QImage::Format_RGB32;
         }
-        QSize outSize(width,height);
-        if (!scaledSize.isEmpty() && quint32(scaledSize.width()) <= width &&
-            quint32(scaledSize.height()) <= height && interlace_method == PNG_INTERLACE_NONE) {
-            // Do inline downscaling
-            outSize = scaledSize;
-            if (doScaledRead)
-                *doScaledRead = true;
-        }
-        if (image.size() != outSize || image.format() != format) {
-            image = QImage(outSize, format);
-            if (image.isNull())
-                return;
-        }
+        if (!QImageIOHandler::allocateImage(size, format, &image))
+            return false;
 
         if (QSysInfo::ByteOrder == QSysInfo::BigEndian)
             png_set_swap_alpha(png_ptr);
@@ -416,91 +347,13 @@ void setup_qt(QImage& image, png_structp png_ptr, png_infop info_ptr, QSize scal
 
         png_read_update_info(png_ptr, info_ptr);
     }
-}
-
-static void read_image_scaled(QImage *outImage, png_structp png_ptr, png_infop info_ptr,
-                              QPngHandlerPrivate::AllocatedMemoryPointers &amp, QSize scaledSize)
-{
-
-    png_uint_32 width = 0;
-    png_uint_32 height = 0;
-    png_int_32 offset_x = 0;
-    png_int_32 offset_y = 0;
-
-    int bit_depth = 0;
-    int color_type = 0;
-    int unit_type = PNG_OFFSET_PIXEL;
-    png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, 0, 0, 0);
-    png_get_oFFs(png_ptr, info_ptr, &offset_x, &offset_y, &unit_type);
-    uchar *data = outImage->bits();
-    int bpl = outImage->bytesPerLine();
-
-    if (scaledSize.isEmpty() || !width || !height)
-        return;
-
-    const quint32 iysz = height;
-    const quint32 ixsz = width;
-    const quint32 oysz = scaledSize.height();
-    const quint32 oxsz = scaledSize.width();
-    const quint32 ibw = 4*width;
-    amp.accRow = new quint32[ibw];
-    memset(amp.accRow, 0, ibw*sizeof(quint32));
-    amp.inRow = new png_byte[ibw];
-    memset(amp.inRow, 0, ibw*sizeof(png_byte));
-    amp.outRow = new uchar[ibw];
-    memset(amp.outRow, 0, ibw*sizeof(uchar));
-    qint32 rval = 0;
-    for (quint32 oy=0; oy<oysz; oy++) {
-        // Store the rest of the previous input row, if any
-        for (quint32 i=0; i < ibw; i++)
-            amp.accRow[i] = rval*amp.inRow[i];
-        // Accumulate the next input rows
-        for (rval = iysz-rval; rval > 0; rval-=oysz) {
-            png_read_row(png_ptr, amp.inRow, NULL);
-            quint32 fact = qMin(oysz, quint32(rval));
-            for (quint32 i=0; i < ibw; i++)
-                amp.accRow[i] += fact*amp.inRow[i];
-        }
-        rval *= -1;
-
-        // We have a full output row, store it
-        for (quint32 i=0; i < ibw; i++)
-            amp.outRow[i] = uchar(amp.accRow[i]/iysz);
-
-        quint32 a[4] = {0, 0, 0, 0};
-        qint32 cval = oxsz;
-        quint32 ix = 0;
-        for (quint32 ox=0; ox<oxsz; ox++) {
-            for (quint32 i=0; i < 4; i++)
-                a[i] = cval * amp.outRow[ix+i];
-            for (cval = ixsz - cval; cval > 0; cval-=oxsz) {
-                ix += 4;
-                if (ix >= ibw)
-                    break;            // Safety belt, should not happen
-                quint32 fact = qMin(oxsz, quint32(cval));
-                for (quint32 i=0; i < 4; i++)
-                    a[i] += fact * amp.outRow[ix+i];
-            }
-            cval *= -1;
-            for (quint32 i=0; i < 4; i++)
-                data[(4*ox)+i] = uchar(a[i]/ixsz);
-        }
-        data += bpl;
-    }
-    amp.deallocate();
-
-    outImage->setDotsPerMeterX((png_get_x_pixels_per_meter(png_ptr,info_ptr)*oxsz)/ixsz);
-    outImage->setDotsPerMeterY((png_get_y_pixels_per_meter(png_ptr,info_ptr)*oysz)/iysz);
-
-    if (unit_type == PNG_OFFSET_PIXEL)
-        outImage->setOffset(QPoint(offset_x*oxsz/ixsz, offset_y*oysz/iysz));
-
+    return true;
 }
 
 extern "C" {
 static void qt_png_warning(png_structp /*png_ptr*/, png_const_charp message)
 {
-    qWarning("libpng warning: %s", message);
+    qCInfo(lcImageIo, "libpng warning: %s", message);
 }
 
 }
@@ -508,6 +361,7 @@ static void qt_png_warning(png_structp /*png_ptr*/, png_const_charp message)
 
 void QPngHandlerPrivate::readPngTexts(png_info *info)
 {
+#ifndef QT_NO_IMAGEIO_TEXT_LOADING
     png_textp text_ptr;
     int num_text=0;
     png_get_text(png_ptr, info, &text_ptr, &num_text);
@@ -524,23 +378,26 @@ void QPngHandlerPrivate::readPngTexts(png_info *info)
             value = QString::fromLatin1(text_ptr->text, int(text_ptr->text_length));
         }
         if (!description.isEmpty())
-            description += QLatin1String("\n\n");
-        description += key + QLatin1String(": ") + value.simplified();
+            description += "\n\n"_L1;
+        description += key + ": "_L1 + value.simplified();
         readTexts.append(key);
         readTexts.append(value);
         text_ptr++;
     }
+#else
+    Q_UNUSED(info);
+#endif
 }
 
 
 bool QPngHandlerPrivate::readPngHeader()
 {
     state = Error;
-    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,0,0,0);
+    png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,nullptr,nullptr,nullptr);
     if (!png_ptr)
         return false;
 
-    png_set_error_fn(png_ptr, 0, 0, qt_png_warning);
+    png_set_error_fn(png_ptr, nullptr, nullptr, qt_png_warning);
 
 #if defined(PNG_SET_OPTION_SUPPORTED) && defined(PNG_MAXIMUM_INFLATE_WINDOW)
     // Trade off a little bit of memory for better compatibility with existing images
@@ -550,21 +407,21 @@ bool QPngHandlerPrivate::readPngHeader()
 
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) {
-        png_destroy_read_struct(&png_ptr, 0, 0);
-        png_ptr = 0;
+        png_destroy_read_struct(&png_ptr, nullptr, nullptr);
+        png_ptr = nullptr;
         return false;
     }
 
     end_info = png_create_info_struct(png_ptr);
     if (!end_info) {
-        png_destroy_read_struct(&png_ptr, &info_ptr, 0);
-        png_ptr = 0;
+        png_destroy_read_struct(&png_ptr, &info_ptr, nullptr);
+        png_ptr = nullptr;
         return false;
     }
 
     if (setjmp(png_jmpbuf(png_ptr))) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
-        png_ptr = 0;
+        png_ptr = nullptr;
         return false;
     }
 
@@ -573,10 +430,63 @@ bool QPngHandlerPrivate::readPngHeader()
 
     readPngTexts(info_ptr);
 
-    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_gAMA)) {
+#ifdef PNG_iCCP_SUPPORTED
+    if (png_get_valid(png_ptr, info_ptr, PNG_INFO_iCCP)) {
+        png_charp name = nullptr;
+        int compressionType = 0;
+#if (PNG_LIBPNG_VER < 10500)
+        png_charp profileData = nullptr;
+#else
+        png_bytep profileData = nullptr;
+#endif
+        png_uint_32 profLen;
+        png_get_iCCP(png_ptr, info_ptr, &name, &compressionType, &profileData, &profLen);
+        Q_UNUSED(name);
+        Q_UNUSED(compressionType);
+        if (profLen > 0) {
+            colorSpace = QColorSpace::fromIccProfile(QByteArray((const char *)profileData, profLen));
+            QColorSpacePrivate *csD = QColorSpacePrivate::get(colorSpace);
+            if (csD->description.isEmpty())
+                csD->description = QString::fromLatin1((const char *)name);
+            colorSpaceState = Icc;
+        }
+    }
+#endif
+    if (colorSpaceState <= Srgb && png_get_valid(png_ptr, info_ptr, PNG_INFO_sRGB)) {
+        int rendering_intent = -1;
+        png_get_sRGB(png_ptr, info_ptr, &rendering_intent);
+        // We don't actually care about the rendering_intent, just that it is valid
+        if (rendering_intent >= 0 && rendering_intent <= 3) {
+            colorSpace = QColorSpace::SRgb;
+            colorSpaceState = Srgb;
+        }
+    }
+    if (colorSpaceState <= GammaChrm && png_get_valid(png_ptr, info_ptr, PNG_INFO_gAMA)) {
         double file_gamma = 0.0;
         png_get_gAMA(png_ptr, info_ptr, &file_gamma);
         fileGamma = file_gamma;
+        if (fileGamma > 0.0f) {
+            QColorSpace::PrimaryPoints primaries;
+            if (png_get_valid(png_ptr, info_ptr, PNG_INFO_cHRM)) {
+                double white_x, white_y, red_x, red_y;
+                double green_x, green_y, blue_x, blue_y;
+                png_get_cHRM(png_ptr, info_ptr,
+                             &white_x, &white_y, &red_x, &red_y,
+                             &green_x, &green_y, &blue_x, &blue_y);
+                primaries.whitePoint = QPointF(white_x, white_y);
+                primaries.redPoint = QPointF(red_x, red_y);
+                primaries.greenPoint = QPointF(green_x, green_y);
+                primaries.bluePoint = QPointF(blue_x, blue_y);
+            }
+            if (primaries.isValid()) {
+                colorSpace = QColorSpace(primaries.whitePoint, primaries.redPoint, primaries.greenPoint, primaries.bluePoint,
+                                         QColorSpace::TransferFunction::Gamma, 1.0f / fileGamma);
+            } else {
+                colorSpace = QColorSpace(QColorSpace::Primaries::SRgb,
+                                         QColorSpace::TransferFunction::Gamma, 1.0f / fileGamma);
+            }
+            colorSpaceState = GammaChrm;
+        }
     }
 
     state = ReadHeader;
@@ -593,65 +503,66 @@ bool QPngHandlerPrivate::readPngImage(QImage *outImage)
         return false;
     }
 
+    row_pointers = nullptr;
     if (setjmp(png_jmpbuf(png_ptr))) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
-        png_ptr = 0;
-        amp.deallocate();
+        png_ptr = nullptr;
+        delete[] row_pointers;
         state = Error;
         return false;
     }
 
-    bool doScaledRead = false;
-    setup_qt(*outImage, png_ptr, info_ptr, scaledSize, &doScaledRead, gamma, fileGamma);
+    if (gamma != 0.0 && fileGamma != 0.0) {
+        // This configuration forces gamma correction and
+        // thus changes the output colorspace
+        png_set_gamma(png_ptr, 1.0f / gamma, fileGamma);
+        colorSpace.setTransferFunction(QColorSpace::TransferFunction::Gamma, 1.0f / gamma);
+        colorSpaceState = GammaChrm;
+    }
 
-    if (outImage->isNull()) {
+    if (!setup_qt(*outImage, png_ptr, info_ptr)) {
         png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
-        png_ptr = 0;
-        amp.deallocate();
+        png_ptr = nullptr;
+        delete[] row_pointers;
         state = Error;
         return false;
     }
 
-    if (doScaledRead) {
-        read_image_scaled(outImage, png_ptr, info_ptr, amp, scaledSize);
-    } else {
-        png_uint_32 width = 0;
-        png_uint_32 height = 0;
-        png_int_32 offset_x = 0;
-        png_int_32 offset_y = 0;
+    png_uint_32 width = 0;
+    png_uint_32 height = 0;
+    png_int_32 offset_x = 0;
+    png_int_32 offset_y = 0;
 
-        int bit_depth = 0;
-        int color_type = 0;
-        int unit_type = PNG_OFFSET_PIXEL;
-        png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, 0, 0, 0);
-        png_get_oFFs(png_ptr, info_ptr, &offset_x, &offset_y, &unit_type);
-        uchar *data = outImage->bits();
-        int bpl = outImage->bytesPerLine();
-        amp.row_pointers = new png_bytep[height];
+    int bit_depth = 0;
+    int color_type = 0;
+    int unit_type = PNG_OFFSET_PIXEL;
+    png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, nullptr, nullptr, nullptr);
+    png_get_oFFs(png_ptr, info_ptr, &offset_x, &offset_y, &unit_type);
+    uchar *data = outImage->bits();
+    qsizetype bpl = outImage->bytesPerLine();
+    row_pointers = new png_bytep[height];
 
-        for (uint y = 0; y < height; y++)
-            amp.row_pointers[y] = data + y * bpl;
+    for (uint y = 0; y < height; y++)
+        row_pointers[y] = data + y * bpl;
 
-        png_read_image(png_ptr, amp.row_pointers);
-        amp.deallocate();
+    png_read_image(png_ptr, row_pointers);
 
-        outImage->setDotsPerMeterX(png_get_x_pixels_per_meter(png_ptr,info_ptr));
-        outImage->setDotsPerMeterY(png_get_y_pixels_per_meter(png_ptr,info_ptr));
+    outImage->setDotsPerMeterX(png_get_x_pixels_per_meter(png_ptr,info_ptr));
+    outImage->setDotsPerMeterY(png_get_y_pixels_per_meter(png_ptr,info_ptr));
 
-        if (unit_type == PNG_OFFSET_PIXEL)
-            outImage->setOffset(QPoint(offset_x, offset_y));
+    if (unit_type == PNG_OFFSET_PIXEL)
+        outImage->setOffset(QPoint(offset_x, offset_y));
 
-        // sanity check palette entries
-        if (color_type == PNG_COLOR_TYPE_PALETTE && outImage->format() == QImage::Format_Indexed8) {
-            int color_table_size = outImage->colorCount();
-            for (int y=0; y<(int)height; ++y) {
-                uchar *p = FAST_SCAN_LINE(data, bpl, y);
-                uchar *end = p + width;
-                while (p < end) {
-                    if (*p >= color_table_size)
-                        *p = 0;
-                    ++p;
-                }
+           // sanity check palette entries
+    if (color_type == PNG_COLOR_TYPE_PALETTE && outImage->format() == QImage::Format_Indexed8) {
+        int color_table_size = outImage->colorCount();
+        for (int y=0; y<(int)height; ++y) {
+            uchar *p = FAST_SCAN_LINE(data, bpl, y);
+            uchar *end = p + width;
+            while (p < end) {
+                if (*p >= color_table_size)
+                    *p = 0;
+                ++p;
             }
         }
     }
@@ -664,12 +575,13 @@ bool QPngHandlerPrivate::readPngImage(QImage *outImage)
         outImage->setText(readTexts.at(i), readTexts.at(i+1));
 
     png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
-    png_ptr = 0;
-    amp.deallocate();
+    png_ptr = nullptr;
+    delete[] row_pointers;
+    row_pointers = nullptr;
     state = Ready;
 
-    if (scaledSize.isValid() && outImage->size() != scaledSize)
-        *outImage = outImage->scaled(scaledSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+    if (colorSpaceState > Undefined && colorSpace.isValid())
+        outImage->setColorSpace(colorSpace);
 
     return true;
 }
@@ -677,17 +589,17 @@ bool QPngHandlerPrivate::readPngImage(QImage *outImage)
 QImage::Format QPngHandlerPrivate::readImageFormat()
 {
         QImage::Format format = QImage::Format_Invalid;
-        png_uint_32 width, height;
+        png_uint_32 width = 0, height = 0;
         int bit_depth = 0, color_type = 0;
         png_colorp palette;
         int num_palette;
-        png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, 0, 0, 0);
+        png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, nullptr, nullptr, nullptr);
         if (color_type == PNG_COLOR_TYPE_GRAY) {
             // Black & White or grayscale
             if (bit_depth == 1 && png_get_channels(png_ptr, info_ptr) == 1) {
                 format = QImage::Format_Mono;
             } else if (bit_depth == 16) {
-                format = png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS) ? QImage::Format_RGBA64 : QImage::Format_RGBX64;
+                format = png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS) ? QImage::Format_RGBA64 : QImage::Format_Grayscale16;
             } else if (bit_depth == 8 && !png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) {
                 format = QImage::Format_Grayscale8;
             } else {
@@ -765,12 +677,12 @@ static void set_text(const QImage &image, png_structp png_ptr, png_infop info_pt
     QMap<QString, QString>::ConstIterator it = text.constBegin();
     int i = 0;
     while (it != text.constEnd()) {
-        text_ptr[i].key = qstrdup(it.key().leftRef(79).toLatin1().constData());
-        bool noCompress = (it.value().length() < 40);
+        text_ptr[i].key = qstrdup(QStringView{it.key()}.left(79).toLatin1().constData());
+        bool noCompress = (it.value().size() < 40);
 
 #ifdef PNG_iTXt_SUPPORTED
         bool needsItxt = false;
-        for (const QChar c : it.value()) {
+        for (QChar c : it.value()) {
             uchar ch = c.cell();
             if (c.row() || (ch < 0x20 && ch != '\n') || (ch > 0x7e && ch < 0xa0)) {
                 needsItxt = true;
@@ -814,7 +726,7 @@ bool QPNGImageWriter::writeImage(const QImage& image, int off_x, int off_y)
     return writeImage(image, -1, QString(), off_x, off_y);
 }
 
-bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_in, const QString &description,
+bool QPNGImageWriter::writeImage(const QImage& image, int compression_in, const QString &description,
                                  int off_x_in, int off_y_in)
 {
     QPoint offset = image.offset();
@@ -824,16 +736,19 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
     png_structp png_ptr;
     png_infop info_ptr;
 
-    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,0,0,0);
+    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,nullptr,nullptr,nullptr);
     if (!png_ptr) {
         return false;
     }
 
-    png_set_error_fn(png_ptr, 0, 0, qt_png_warning);
+    png_set_error_fn(png_ptr, nullptr, nullptr, qt_png_warning);
+#ifdef PNG_BENIGN_ERRORS_SUPPORTED
+    png_set_benign_errors(png_ptr, 1);
+#endif
 
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) {
-        png_destroy_write_struct(&png_ptr, 0);
+        png_destroy_write_struct(&png_ptr, nullptr);
         return false;
     }
 
@@ -845,7 +760,7 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
     int compression = compression_in;
     if (compression >= 0) {
         if (compression > 9) {
-            qWarning("PNG: Compression %d out of range", compression);
+            qCWarning(lcImageIo, "PNG: Compression %d out of range", compression);
             compression = 9;
         }
         png_set_compression_level(png_ptr, compression);
@@ -861,7 +776,8 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
         else
             color_type = PNG_COLOR_TYPE_PALETTE;
     }
-    else if (image.format() == QImage::Format_Grayscale8)
+    else if (image.format() == QImage::Format_Grayscale8
+             || image.format() == QImage::Format_Grayscale16)
         color_type = PNG_COLOR_TYPE_GRAY;
     else if (image.hasAlphaChannel())
         color_type = PNG_COLOR_TYPE_RGB_ALPHA;
@@ -877,6 +793,7 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
     case QImage::Format_RGBX64:
     case QImage::Format_RGBA64:
     case QImage::Format_RGBA64_Premultiplied:
+    case QImage::Format_Grayscale16:
         bpc = 16;
         break;
     default:
@@ -888,6 +805,26 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
                  bpc, // per channel
                  color_type, 0, 0, 0);       // sets #channels
 
+#ifdef PNG_iCCP_SUPPORTED
+    QColorSpace cs = image.colorSpace();
+    // Support the old gamma making it override transferfunction (if possible)
+    if (cs.isValid() && gamma != 0.0 && !qFuzzyCompare(cs.gamma(), 1.0f / gamma))
+        cs = cs.withTransferFunction(QColorSpace::TransferFunction::Gamma, 1.0f / gamma);
+    QByteArray iccProfile = cs.iccProfile();
+    if (!iccProfile.isEmpty()) {
+        QByteArray iccProfileName = cs.description().toLatin1();
+        if (iccProfileName.isEmpty())
+            iccProfileName = QByteArrayLiteral("Custom");
+        png_set_iCCP(png_ptr, info_ptr,
+             #if PNG_LIBPNG_VER < 10500
+                     iccProfileName.data(), PNG_COMPRESSION_TYPE_BASE, iccProfile.data(),
+             #else
+                     iccProfileName.constData(), PNG_COMPRESSION_TYPE_BASE,
+                     (png_const_bytep)iccProfile.constData(),
+             #endif
+                     iccProfile.size());
+    } else
+#endif
     if (gamma != 0.0) {
         png_set_gAMA(png_ptr, info_ptr, 1.0/gamma);
     }
@@ -914,7 +851,7 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
         png_set_PLTE(png_ptr, info_ptr, palette, num_palette);
 
         if (num_trans) {
-            png_set_tRNS(png_ptr, info_ptr, trans, num_trans, 0);
+            png_set_tRNS(png_ptr, info_ptr, trans, num_trans, nullptr);
         }
     }
 
@@ -971,6 +908,7 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
     if (color_type == PNG_COLOR_TYPE_RGB) {
         switch (image.format()) {
         case QImage::Format_RGB888:
+        case QImage::Format_BGR888:
             break;
         case QImage::Format_RGBX8888:
         case QImage::Format_RGBX64:
@@ -988,6 +926,7 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
         case QImage::Format_RGBX64:
         case QImage::Format_RGBA64:
         case QImage::Format_RGBA64_Premultiplied:
+        case QImage::Format_Grayscale16:
             png_set_swap(png_ptr);
             break;
         default:
@@ -1018,9 +957,11 @@ bool QPNGImageWriter::writeImage(const QImage& image, volatile int compression_i
     case QImage::Format_MonoLSB:
     case QImage::Format_Indexed8:
     case QImage::Format_Grayscale8:
+    case QImage::Format_Grayscale16:
     case QImage::Format_RGB32:
     case QImage::Format_ARGB32:
     case QImage::Format_RGB888:
+    case QImage::Format_BGR888:
     case QImage::Format_RGBX8888:
     case QImage::Format_RGBA8888:
     case QImage::Format_RGBX64:
@@ -1112,7 +1053,7 @@ bool QPngHandler::canRead() const
 bool QPngHandler::canRead(QIODevice *device)
 {
     if (!device) {
-        qWarning("QPngHandler::canRead() called with no device");
+        qCWarning(lcImageIo, "QPngHandler::canRead() called with no device");
         return false;
     }
 
@@ -1138,8 +1079,7 @@ bool QPngHandler::supportsOption(ImageOption option) const
         || option == ImageFormat
         || option == Quality
         || option == CompressionRatio
-        || option == Size
-        || option == ScaledSize;
+        || option == Size;
 }
 
 QVariant QPngHandler::option(ImageOption option) const
@@ -1160,8 +1100,6 @@ QVariant QPngHandler::option(ImageOption option) const
     else if (option == Size)
         return QSize(png_get_image_width(d->png_ptr, d->info_ptr),
                      png_get_image_height(d->png_ptr, d->info_ptr));
-    else if (option == ScaledSize)
-        return d->scaledSize;
     else if (option == ImageFormat)
         return d->readImageFormat();
     return QVariant();
@@ -1177,13 +1115,6 @@ void QPngHandler::setOption(ImageOption option, const QVariant &value)
         d->compression = value.toInt();
     else if (option == Description)
         d->description = value.toString();
-    else if (option == ScaledSize)
-        d->scaledSize = value.toSize();
-}
-
-QByteArray QPngHandler::name() const
-{
-    return "png";
 }
 
 QT_END_NAMESPACE
