@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2013 Klaralvdalens Datakonsult AB (KDAB).
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtOpenGL module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2013 Klaralvdalens Datakonsult AB (KDAB).
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QOPENGLABSTRACTTEXTURE_H
 #define QOPENGLABSTRACTTEXTURE_H
@@ -522,7 +486,7 @@ public:
     void setMipMaxLevel(int maxLevel);
     int mipMaxLevel() const;
     void setMipLevelRange(int baseLevel, int maxLevel);
-    QPair<int, int> mipLevelRange() const;
+    std::pair<int, int> mipLevelRange() const;
 
     void setAutoMipMapGenerationEnabled(bool enabled);
     bool isAutoMipMapGenerationEnabled() const;
@@ -550,9 +514,14 @@ public:
         CompareLess         = 0x0201,   // GL_LESS
         CompareGreater      = 0x0204,   // GL_GREATER
         CompareEqual        = 0x0202,   // GL_EQUAL
-        CommpareNotEqual    = 0x0205,   // GL_NOTEQUAL
+        CompareNotEqual     = 0x0205,   // GL_NOTEQUAL
         CompareAlways       = 0x0207,   // GL_ALWAYS
-        CompareNever        = 0x0200    // GL_NEVER
+        CompareNever        = 0x0200,   // GL_NEVER
+
+#if QT_DEPRECATED_SINCE(6, 1)
+        CommpareNotEqual Q_DECL_ENUMERATOR_DEPRECATED_X(
+            "Use CompareNotEqual instead.") = CompareNotEqual
+#endif
     };
     Q_ENUM(ComparisonFunction)
 
@@ -584,7 +553,7 @@ public:
     Filter magnificationFilter() const;
     void setMinMagFilters(Filter minificationFilter,
                           Filter magnificationFilter);
-    QPair<Filter, Filter> minMagFilters() const;
+    std::pair<Filter, Filter> minMagFilters() const;
     void setMaximumAnisotropy(float anisotropy);
     float maximumAnisotropy() const;
 
@@ -607,7 +576,7 @@ public:
     void setMaximumLevelOfDetail(float value);
     float maximumLevelOfDetail() const;
     void setLevelOfDetailRange(float min, float max);
-    QPair<float, float> levelOfDetailRange() const;
+    std::pair<float, float> levelOfDetailRange() const;
     void setLevelofDetailBias(float bias);
     float levelofDetailBias() const;
 
