@@ -357,11 +357,11 @@ NSMenuItem *QCocoaMenuItem::sync()
 
         QChar cocoaKey = QAppleKeyMapper::toCocoaKey(key);
         if (cocoaKey.isNull())
-            cocoaKey = QChar(key).toLower().unicode();
+            cocoaKey = char16_t(QChar::toLower(key));
         // Similar to qt_mac_removePrivateUnicode change the delete key,
         // so the symbol is correctly seen in native menu bar.
         if (cocoaKey.unicode() == NSDeleteFunctionKey)
-            cocoaKey = QChar(NSDeleteCharacter);
+            cocoaKey = char16_t(NSDeleteCharacter);
 
         m_native.keyEquivalent = QStringView(&cocoaKey, 1).toNSString();
         m_native.keyEquivalentModifierMask = QAppleKeyMapper::toCocoaModifiers(modifiers);
