@@ -64,9 +64,6 @@ void QEglFSWindow::create()
 
     m_flags = Created;
 
-    if (window()->type() == Qt::Desktop)
-        return;
-
     // Stop if there is already a window backed by a native window and surface. Additional
     // raster windows will not have their own native window, surface and context. Instead,
     // they will be composited onto the root window's surface.
@@ -132,29 +129,42 @@ void QEglFSWindow::destroy()
     if (!m_flags.testFlag(Created))
         return; // already destroyed
 
-#ifndef QT_NO_OPENGL
-    QOpenGLCompositor::instance()->removeWindow(this);
-#endif
-
     QEglFSScreen *screen = this->screen();
-    if (m_flags.testFlag(HasNativeWindow)) {
 #ifndef QT_NO_OPENGL
+    QOpenGLCompositor *compositor = QOpenGLCompositor::instance();
+    compositor->removeWindow(this);
+    if (compositor->targetWindow() == window()) {
         QEglFSCursor *cursor = qobject_cast<QEglFSCursor *>(screen->cursor());
         if (cursor)
             cursor->resetResources();
-#endif
+
         if (screen->primarySurface() == m_surface)
             screen->setPrimarySurface(EGL_NO_SURFACE);
 
         invalidateSurface();
 
-#ifndef QT_NO_OPENGL
-        QOpenGLCompositor::destroy();
-        if (qt_gl_global_share_context() == m_rasterCompositingContext)
-            qt_gl_set_global_share_context(nullptr);
-        delete m_rasterCompositingContext;
-#endif
+        if (compositor->windows().isEmpty()) {
+            compositor->destroy();
+            if (qt_gl_global_share_context() == m_rasterCompositingContext)
+                qt_gl_set_global_share_context(nullptr);
+            delete m_rasterCompositingContext;
+        } else {
+            auto topWindow = static_cast<QEglFSWindow *>(compositor->windows().last());
+            // Make fullscreen
+            topWindow->setGeometry(screen->rawGeometry());
+            topWindow->resetSurface();
+            screen->setPrimarySurface(topWindow->surface());
+            compositor->setTargetWindow(topWindow->sourceWindow(), screen->rawGeometry());
+        }
     }
+#else
+    if (m_flags.testFlag(HasNativeWindow)) {
+        if (screen->primarySurface() == m_surface)
+            screen->setPrimarySurface(EGL_NO_SURFACE);
+
+        invalidateSurface();
+    }
+#endif
 
     m_flags = { };
 }
@@ -162,8 +172,28 @@ void QEglFSWindow::destroy()
 void QEglFSWindow::invalidateSurface()
 {
     if (m_surface != EGL_NO_SURFACE) {
-        eglDestroySurface(screen()->display(), m_surface);
+        qCDebug(qLcEglDevDebug) << Q_FUNC_INFO << " about to destroy EGLSurface: " << m_surface;
+
+        bool ok = eglDestroySurface(screen()->display(), m_surface);
+
+        if (!ok) {
+            qCWarning(qLcEglDevDebug, "QEglFSWindow::invalidateSurface() eglDestroySurface failed!"
+                                      " Follow-up errors or memory leaks are possible."
+                                      " eglGetError(): %x", eglGetError());
+        }
+
+        if (eglGetCurrentSurface(EGL_READ) == m_surface ||
+                eglGetCurrentSurface(EGL_DRAW) == m_surface) {
+            bool ok = eglMakeCurrent(eglGetCurrentDisplay(), EGL_NO_DISPLAY, EGL_NO_DISPLAY, EGL_NO_CONTEXT);
+            qCDebug(qLcEglDevDebug) << Q_FUNC_INFO << " due to eglDestroySurface on *currently* bound surface"
+                                    << "we just called eglMakeCurrent(..,0,0,0)! It returned: " << ok;
+        }
+
+        if (screen()->primarySurface() == m_surface)
+            screen()->setPrimarySurface(EGL_NO_SURFACE);
+
         m_surface = EGL_NO_SURFACE;
+        m_flags = m_flags & ~Created;
     }
     qt_egl_device_integration()->destroyNativeWindow(m_window);
     m_window = 0;
@@ -240,7 +270,7 @@ void QEglFSWindow::requestActivateWindow()
         QOpenGLCompositor::instance()->moveToTop(this);
 #endif
     QWindow *wnd = window();
-    QWindowSystemInterface::handleWindowActivated(wnd, Qt::ActiveWindowFocusReason);
+    QWindowSystemInterface::handleFocusWindowChanged(wnd, Qt::ActiveWindowFocusReason);
     QWindowSystemInterface::handleExposeEvent(wnd, QRect(QPoint(0, 0), wnd->geometry().size()));
 }
 
@@ -260,7 +290,7 @@ void QEglFSWindow::lower()
 #ifndef QT_NO_OPENGL
     QOpenGLCompositor *compositor = QOpenGLCompositor::instance();
     QList<QOpenGLCompositorWindow *> windows = compositor->windows();
-    if (window()->type() != Qt::Desktop && windows.count() > 1) {
+    if (window()->type() != Qt::Desktop && windows.size() > 1) {
         int idx = windows.indexOf(this);
         if (idx > 0) {
             compositor->changeWindowIndex(this, idx - 1);
