@@ -1,10 +1,14 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 
+#include <QtTest/private/qcomparisontesthelper_p.h>
+
 #include <QtCore/QCoreApplication>
+#if QT_CONFIG(sortfilterproxymodel)
 #include <QtCore/QSortFilterProxyModel>
+#endif
 #include <QtCore/QStringListModel>
 #include <QtGui/QStandardItemModel>
 
@@ -36,6 +40,8 @@ private slots:
     void parent();
     void hasChildren();
     void data();
+    void invalidModelIndexDataReturnsInvalidQVariant();
+    void invalidPersistentModelIndexDataReturnsInvalidQVariant();
     void headerData();
     void itemData();
     void itemFlags();
@@ -56,6 +62,7 @@ private slots:
     void reset();
 
     void complexChangesWithPersistent();
+    void modelIndexComparisons();
 
     void testMoveSameParentUp_data();
     void testMoveSameParentUp();
@@ -77,9 +84,11 @@ private slots:
     void testMoveWithinOwnRange_data();
     void testMoveWithinOwnRange();
 
+#if QT_CONFIG(sortfilterproxymodel)
     void testMoveThroughProxy();
 
     void testReset();
+#endif
 
     void testDataChanged();
 
@@ -156,8 +165,8 @@ QtTestModel::QtTestModel(const QList<QList<QString> > tbl, QObject *parent)
     : QAbstractItemModel(parent), wrongIndex(false)
 {
     table = tbl;
-    rCount = tbl.count();
-    cCount = tbl.at(0).count();
+    rCount = tbl.size();
+    cCount = tbl.at(0).size();
 }
 
 QModelIndex QtTestModel::index(int row, int column, const QModelIndex &parent) const
@@ -191,7 +200,7 @@ bool QtTestModel::insertRows(int row, int count, const QModelIndex &parent)
     QAbstractItemModel::beginInsertRows(parent, row, row + count - 1);
     int cc = columnCount(parent);
     table.insert(row, count, QList<QString>(cc));
-    rCount = table.count();
+    rCount = table.size();
     QAbstractItemModel::endInsertRows();
     return true;
 }
@@ -202,7 +211,7 @@ bool QtTestModel::insertColumns(int column, int count, const QModelIndex &parent
     int rc = rowCount(parent);
     for (int i = 0; i < rc; ++i)
         table[i].insert(column, 1, "");
-    cCount = table.at(0).count();
+    cCount = table.at(0).size();
     QAbstractItemModel::endInsertColumns();
     return true;
 }
@@ -218,7 +227,7 @@ bool QtTestModel::removeRows( int row, int count, const QModelIndex & parent)
 
     for (int r = row+count-1; r >= row; --r)
         table.remove(r);
-    rCount = table.count();
+    rCount = table.size();
 
     QAbstractItemModel::endRemoveRows();
     return true;
@@ -232,7 +241,7 @@ bool QtTestModel::removeColumns(int column, int count, const QModelIndex & paren
         for (int r = 0; r < rCount; ++r)
             table[r].remove(c);
 
-    cCount = table.at(0).count();
+    cCount = table.at(0).size();
 
     QAbstractItemModel::endRemoveColumns();
     return true;
@@ -260,7 +269,7 @@ bool QtTestModel::moveRows(const QModelIndex &sourceParent, int src, int cnt,
         }
     }
 
-    rCount = table.count();
+    rCount = table.size();
 
     QAbstractItemModel::endMoveRows();
     return true;
@@ -290,7 +299,7 @@ bool QtTestModel::moveColumns(const QModelIndex &sourceParent, int src, int cnt,
         }
     }
 
-    cCount = table.at(0).count();
+    cCount = table.at(0).size();
 
     QAbstractItemModel::endMoveColumns();
     return true;
@@ -383,6 +392,33 @@ void tst_QAbstractItemModel::data()
     QCOMPARE(model.setHeaderData(0, Qt::Horizontal, QVariant(0), 0), false);
 }
 
+void tst_QAbstractItemModel::invalidModelIndexDataReturnsInvalidQVariant()
+{
+    const QModelIndex invalid;
+    QVERIFY(!invalid.isValid());
+    QVERIFY(!invalid.data(Qt::ItemDataRole::DisplayRole).isValid());
+
+    QtTestModel model(1, 1);
+    const QModelIndex mi = model.index(0, 1);
+    QVERIFY(!mi.isValid());
+    QVERIFY(!mi.data(Qt::ItemDataRole::DisplayRole).isValid());
+}
+
+void tst_QAbstractItemModel::invalidPersistentModelIndexDataReturnsInvalidQVariant()
+{
+    const QPersistentModelIndex invalid;
+    QVERIFY(!invalid.isValid());
+    QVERIFY(!invalid.data(Qt::ItemDataRole::DisplayRole).isValid());
+
+    QtTestModel model(1, 1);
+    const QPersistentModelIndex pmi = model.index(0, 0);
+    QVERIFY(pmi.isValid());
+    QVERIFY(pmi.data(Qt::ItemDataRole::DisplayRole).isValid());
+    model.removeRows(0, 1);
+    QVERIFY(!pmi.isValid());
+    QVERIFY(!pmi.data(Qt::ItemDataRole::DisplayRole).isValid());
+}
+
 void tst_QAbstractItemModel::headerData()
 {
     QtTestModel model(1, 1);
@@ -414,11 +450,11 @@ void tst_QAbstractItemModel::itemFlags()
 
 void tst_QAbstractItemModel::match()
 {
-    QtTestModel model(4, 1);
+    QtTestModel model(5, 1);
     QModelIndex start = model.index(0, 0, QModelIndex());
     QVERIFY(start.isValid());
     QModelIndexList res = model.match(start, Qt::DisplayRole, QVariant("1"), 3);
-    QCOMPARE(res.count(), 1);
+    QCOMPARE(res.size(), 1);
     QModelIndex idx = model.index(1, 0, QModelIndex());
     bool areEqual = (idx == res.first());
     QVERIFY(areEqual);
@@ -427,44 +463,45 @@ void tst_QAbstractItemModel::match()
     model.setData(model.index(1, 0, QModelIndex()), "cat", Qt::DisplayRole);
     model.setData(model.index(2, 0, QModelIndex()), "dog", Qt::DisplayRole);
     model.setData(model.index(3, 0, QModelIndex()), "boar", Qt::DisplayRole);
+    model.setData(model.index(4, 0, QModelIndex()), "bo/a/r", Qt::DisplayRole); // QTBUG-104585
 
     res = model.match(start, Qt::DisplayRole, QVariant("dog"), -1, Qt::MatchExactly);
-    QCOMPARE(res.count(), 1);
+    QCOMPARE(res.size(), 1);
     res = model.match(start, Qt::DisplayRole, QVariant("a"), -1, Qt::MatchContains);
-    QCOMPARE(res.count(), 3);
+    QCOMPARE(res.size(), 4);
     res = model.match(start, Qt::DisplayRole, QVariant("b"), -1, Qt::MatchStartsWith);
-    QCOMPARE(res.count(), 2);
+    QCOMPARE(res.size(), 3);
     res = model.match(start, Qt::DisplayRole, QVariant("t"), -1, Qt::MatchEndsWith);
-    QCOMPARE(res.count(), 2);
+    QCOMPARE(res.size(), 2);
     res = model.match(start, Qt::DisplayRole, QVariant("*a*"), -1, Qt::MatchWildcard);
-    QCOMPARE(res.count(), 3);
+    QCOMPARE(res.size(), 4);
     res = model.match(start, Qt::DisplayRole, QVariant(".*O.*"), -1, Qt::MatchRegularExpression);
-    QCOMPARE(res.count(), 2);
+    QCOMPARE(res.size(), 3);
     res = model.match(start, Qt::DisplayRole, QVariant(".*O.*"), -1, Qt::MatchRegularExpression | Qt::MatchCaseSensitive);
-    QCOMPARE(res.count(), 0);
+    QCOMPARE(res.size(), 0);
     res = model.match(start, Qt::DisplayRole, QVariant("BOAR"), -1, Qt::MatchFixedString);
-    QCOMPARE(res.count(), 1);
+    QCOMPARE(res.size(), 1);
     res = model.match(start, Qt::DisplayRole, QVariant("bat"), -1,
                       Qt::MatchFixedString | Qt::MatchCaseSensitive);
-    QCOMPARE(res.count(), 1);
+    QCOMPARE(res.size(), 1);
 
     res = model.match(start, Qt::DisplayRole, QVariant(".*O.*"), -1,
                       Qt::MatchRegularExpression);
-    QCOMPARE(res.count(), 2);
+    QCOMPARE(res.size(), 3);
     res = model.match(start, Qt::DisplayRole, QVariant(".*O.*"), -1,
                       Qt::MatchRegularExpression | Qt::MatchCaseSensitive);
-    QCOMPARE(res.count(), 0);
+    QCOMPARE(res.size(), 0);
 
     res = model.match(start, Qt::DisplayRole, QVariant(QRegularExpression(".*O.*")),
                       -1, Qt::MatchRegularExpression);
-    QCOMPARE(res.count(), 0);
+    QCOMPARE(res.size(), 0);
     res = model.match(start,
                       Qt::DisplayRole,
                       QVariant(QRegularExpression(".*O.*",
                                                   QRegularExpression::CaseInsensitiveOption)),
                       -1,
                       Qt::MatchRegularExpression);
-    QCOMPARE(res.count(), 2);
+    QCOMPARE(res.size(), 3);
 
     // Ensure that the case sensitivity is properly ignored when passing a
     // QRegularExpression object.
@@ -474,7 +511,7 @@ void tst_QAbstractItemModel::match()
                                                   QRegularExpression::CaseInsensitiveOption)),
                       -1,
                       Qt::MatchRegularExpression | Qt::MatchCaseSensitive);
-    QCOMPARE(res.count(), 2);
+    QCOMPARE(res.size(), 3);
 }
 
 typedef QPair<int, int> Position;
@@ -757,7 +794,7 @@ void tst_QAbstractItemModel::dropMimeData()
 
     // get the mimeData from the "selected" indexes
     QModelIndexList selectedIndexes;
-    for (int i = 0; i < selection.count(); ++i)
+    for (int i = 0; i < selection.size(); ++i)
         selectedIndexes << src.index(selection.at(i).first, selection.at(i).second, QModelIndex());
     QMimeData *md = src.mimeData(selectedIndexes);
     // do the drop
@@ -828,8 +865,8 @@ void tst_QAbstractItemModel::removeRows()
     QVERIFY(rowsRemovedSpy.isValid());
 
     QCOMPARE(model.removeRows(6, 4), true);
-    QCOMPARE(rowsAboutToBeRemovedSpy.count(), 1);
-    QCOMPARE(rowsRemovedSpy.count(), 1);
+    QCOMPARE(rowsAboutToBeRemovedSpy.size(), 1);
+    QCOMPARE(rowsRemovedSpy.size(), 1);
 }
 
 void tst_QAbstractItemModel::removeColumns()
@@ -843,8 +880,8 @@ void tst_QAbstractItemModel::removeColumns()
     QVERIFY(columnsRemovedSpy.isValid());
 
     QCOMPARE(model.removeColumns(6, 4), true);
-    QCOMPARE(columnsAboutToBeRemovedSpy.count(), 1);
-    QCOMPARE(columnsRemovedSpy.count(), 1);
+    QCOMPARE(columnsAboutToBeRemovedSpy.size(), 1);
+    QCOMPARE(columnsRemovedSpy.size(), 1);
 }
 
 void tst_QAbstractItemModel::insertRows()
@@ -858,8 +895,8 @@ void tst_QAbstractItemModel::insertRows()
     QVERIFY(rowsInsertedSpy.isValid());
 
     QCOMPARE(model.insertRows(6, 4), true);
-    QCOMPARE(rowsAboutToBeInsertedSpy.count(), 1);
-    QCOMPARE(rowsInsertedSpy.count(), 1);
+    QCOMPARE(rowsAboutToBeInsertedSpy.size(), 1);
+    QCOMPARE(rowsInsertedSpy.size(), 1);
 }
 
 void tst_QAbstractItemModel::insertColumns()
@@ -873,8 +910,8 @@ void tst_QAbstractItemModel::insertColumns()
     QVERIFY(columnsInsertedSpy.isValid());
 
     QCOMPARE(model.insertColumns(6, 4), true);
-    QCOMPARE(columnsAboutToBeInsertedSpy.count(), 1);
-    QCOMPARE(columnsInsertedSpy.count(), 1);
+    QCOMPARE(columnsAboutToBeInsertedSpy.size(), 1);
+    QCOMPARE(columnsInsertedSpy.size(), 1);
 }
 
 void tst_QAbstractItemModel::moveRows()
@@ -888,8 +925,8 @@ void tst_QAbstractItemModel::moveRows()
     QVERIFY(rowsMovedSpy.isValid());
 
     QCOMPARE(model.moveRows(QModelIndex(), 6, 4, QModelIndex(), 1), true);
-    QCOMPARE(rowsAboutToBeMovedSpy.count(), 1);
-    QCOMPARE(rowsMovedSpy.count(), 1);
+    QCOMPARE(rowsAboutToBeMovedSpy.size(), 1);
+    QCOMPARE(rowsMovedSpy.size(), 1);
 }
 
 void tst_QAbstractItemModel::moveColumns()
@@ -903,12 +940,12 @@ void tst_QAbstractItemModel::moveColumns()
     QVERIFY(columnsMovedSpy.isValid());
 
     QCOMPARE(model.moveColumns(QModelIndex(), 6, 4, QModelIndex(), 1), true);
-    QCOMPARE(columnsAboutToBeMovedSpy.count(), 1);
-    QCOMPARE(columnsMovedSpy.count(), 1);
+    QCOMPARE(columnsAboutToBeMovedSpy.size(), 1);
+    QCOMPARE(columnsMovedSpy.size(), 1);
 
     QCOMPARE(model.moveColumn(QModelIndex(), 4, QModelIndex(), 1), true);
-    QCOMPARE(columnsAboutToBeMovedSpy.count(), 2);
-    QCOMPARE(columnsMovedSpy.count(), 2);
+    QCOMPARE(columnsAboutToBeMovedSpy.size(), 2);
+    QCOMPARE(columnsMovedSpy.size(), 2);
 }
 
 void tst_QAbstractItemModel::reset()
@@ -918,7 +955,7 @@ void tst_QAbstractItemModel::reset()
     QSignalSpy resetSpy(&model, &QtTestModel::modelReset);
     QVERIFY(resetSpy.isValid());
     model.reset();
-    QCOMPARE(resetSpy.count(), 1);
+    QCOMPARE(resetSpy.size(), 1);
 }
 
 void tst_QAbstractItemModel::complexChangesWithPersistent()
@@ -983,6 +1020,42 @@ void tst_QAbstractItemModel::complexChangesWithPersistent()
         QVERIFY(e[i] == model.index(2, i-2 , QModelIndex()));
 }
 
+void tst_QAbstractItemModel::modelIndexComparisons()
+{
+    QTestPrivate::testAllComparisonOperatorsCompile<QModelIndex>();
+    QTestPrivate::testAllComparisonOperatorsCompile<QPersistentModelIndex>();
+    QTestPrivate::testAllComparisonOperatorsCompile<QPersistentModelIndex, QModelIndex>();
+
+    QtTestModel model(3, 3);
+
+    QModelIndex mi11 = model.index(1, 1);
+    QModelIndex mi22 = model.index(2, 2);
+    QPersistentModelIndex pmi11 = mi11;
+    QPersistentModelIndex pmi22 = mi22;
+    QPersistentModelIndex pmiU;
+
+    QT_TEST_EQUALITY_OPS(mi11, mi11, true);
+    QT_TEST_EQUALITY_OPS(mi11, mi22, false);
+    QT_TEST_ALL_COMPARISON_OPS(mi11, mi11, Qt::strong_ordering::equal);
+    QT_TEST_ALL_COMPARISON_OPS(mi11, mi22, Qt::strong_ordering::less);
+    QT_TEST_ALL_COMPARISON_OPS(mi22, mi11, Qt::strong_ordering::greater);
+    QT_TEST_EQUALITY_OPS(pmi11, pmi11, true);
+    QT_TEST_EQUALITY_OPS(pmi11, pmi22, false);
+    QT_TEST_EQUALITY_OPS(pmi11, mi11, true);
+    QT_TEST_EQUALITY_OPS(pmi11, mi22, false);
+
+    QT_TEST_ALL_COMPARISON_OPS(pmi11, pmi11, Qt::strong_ordering::equal);
+    QT_TEST_ALL_COMPARISON_OPS(pmi11, pmi22, Qt::strong_ordering::less);
+    // Disengaged QPMIs are sorted randomly (based on address of their Private)
+    // So all we can check here is QPMIs with d == nullptr, which should reliably
+    // come before any others.
+    QT_TEST_ALL_COMPARISON_OPS(pmiU, pmiU, Qt::strong_ordering::equal);
+    QT_TEST_ALL_COMPARISON_OPS(pmi11, pmiU, Qt::strong_ordering::greater);
+    QT_TEST_ALL_COMPARISON_OPS(pmi11, mi11, Qt::strong_ordering::equal);
+    QT_TEST_ALL_COMPARISON_OPS(pmi11, mi22, Qt::strong_ordering::less);
+    QT_TEST_ALL_COMPARISON_OPS(pmiU, mi11, Qt::strong_ordering::less);
+}
+
 void tst_QAbstractItemModel::testMoveSameParentDown_data()
 {
     QTest::addColumn<int>("startRow");
@@ -1214,6 +1287,7 @@ void tst_QAbstractItemModel::testMoveSameParentUp()
     }
 }
 
+#if QT_CONFIG(sortfilterproxymodel)
 void tst_QAbstractItemModel::testMoveThroughProxy()
 {
     QSortFilterProxyModel *proxy = new QSortFilterProxyModel(this);
@@ -1232,6 +1306,7 @@ void tst_QAbstractItemModel::testMoveThroughProxy()
     moveCommand->setDestRow(0);
     moveCommand->doCommand();
 }
+#endif
 
 void tst_QAbstractItemModel::testMoveToGrandParent_data()
 {
@@ -1779,6 +1854,7 @@ void tst_QAbstractItemModel::testMoveWithinOwnRange()
     QCOMPARE(afterSpy.size(), 0);
 }
 
+#if QT_CONFIG(proxymodel)
 class ListenerObject : public QObject
 {
     Q_OBJECT
@@ -1797,7 +1873,7 @@ private:
     QList<QPersistentModelIndex> m_persistentIndexes;
     QModelIndexList m_nonPersistentIndexes;
 };
-
+#endif
 
 class ModelWithCustomRole : public QStringListModel
 {
@@ -1811,6 +1887,7 @@ public:
     }
 };
 
+#if QT_CONFIG(proxymodel)
 ListenerObject::ListenerObject(QAbstractProxyModel *parent)
     : QObject(parent), m_model(parent)
 {
@@ -1847,11 +1924,13 @@ void ListenerObject::slotAboutToBeReset()
 
 void ListenerObject::slotReset()
 {
-    for (const auto &idx : qAsConst(m_persistentIndexes)) {
+    for (const auto &idx : std::as_const(m_persistentIndexes)) {
         QVERIFY(!idx.isValid());
     }
 }
+#endif
 
+#if QT_CONFIG(sortfilterproxymodel)
 void tst_QAbstractItemModel::testReset()
 {
     QSignalSpy beforeResetSpy(m_model, &DynamicTreeModel::modelAboutToBeReset);
@@ -1906,6 +1985,7 @@ void tst_QAbstractItemModel::testReset()
     // After being reset the proxy must be queried again.
     QCOMPARE(nullProxy->roleNames().value(Qt::UserRole + 1), QByteArray());
 }
+#endif
 
 class CustomRoleModel : public QStringListModel
 {
