@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QtGui>
@@ -103,7 +103,7 @@ QStringList tst_QAccessibilityLinux::getChildren(QDBusInterface *interface)
 
     Q_ASSERT(interface->property("ChildCount").toInt() == list.count());
     QStringList children;
-    Q_FOREACH (const QSpiObjectReference &ref, list)
+    for (const QSpiObjectReference &ref : std::as_const(list))
         children << ref.path.path();
 
     return children;
